import ApplicationComponent from "../ApplicationComponent";
import express, {Express, Router} from "express";
import Logger from "../Logger";
import {Server} from "http";
import compression from "compression";

export default class ExpressAppComponent extends ApplicationComponent<void> {
    private readonly port: number;
    private server?: Server;

    constructor(port: number) {
        super();
        this.port = port;
    }

    public async start(app: Express): Promise<void> {
        this.server = app.listen(this.port, 'localhost', () => {
            Logger.info(`Web server running on localhost:${this.port}.`);
        });
    }

    public async init(router: Router): Promise<void> {
        router.use(express.json());
        router.use(express.urlencoded({
            extended: true,
        }));

        // gzip
        router.use(compression());

        router.use((req, res, next) => {
            req.models = {};
            req.modelCollections = {};
            next();
        });
    }

    public async stop(): Promise<void> {
        if (this.server) {
            await this.close('Webserver', this.server, this.server.close);
        }
    }

    public getServer(): Server {
        if (!this.server) throw 'Server was not initialized.';
        return this.server;
    }
}
