import {Connection} from "mysql";
import MysqlConnectionManager from "./MysqlConnectionManager";

export default abstract class Migration {
    public readonly version: number;

    constructor(version: number) {
        this.version = version;
    }

    async shouldRun(currentVersion: number): Promise<boolean> {
        return this.version > currentVersion;
    }

    abstract async install(connection: Connection): Promise<void>;

    abstract async rollback(connection: Connection): Promise<void>;

    protected async query(queryString: string, connection: Connection): Promise<void> {
        await MysqlConnectionManager.query(queryString, undefined, connection);
    }
}